@@ -1,5 +1,5 @@
 /*
-** Copyright 2000-2004 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2004 S. Varshavchik.  See COPYING for
 ** distribution information.
 */
 
@@ -59,7 +59,7 @@ int auth_passwd(const char *service,
 	if (authdaemondopasswd(buf, strlen(buf)))
 	{
 		free(buf);
-		sleep(5);
+		/*sleep(5);*/
 		return (-1);
 	}
 	free(buf);
