@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2011 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2011 S. Varshavchik.  See COPYING for
 ** distribution information.
 */
 
@@ -27,7 +27,8 @@ static char *a_rfc2047_encode_str(const char *str, const char *charset,
 
 static void rfc2047_encode_header_do(const struct rfc822a *a,
 				     const char *charset,
-				     void (*print_func)(char, void *),
+				     void (*print_func)(const char *, size_t,
+							void *),
 				     void (*print_separator)(const char *,
 							     void *), void *ptr)
 {
@@ -169,9 +170,9 @@ static char *a_rfc2047_encode_str(const char *str, const char *charset,
 	return rfc2047_encode_str(str, charset, rfc2047_qp_allow_word);
 }
 
-static void count(char c, void *p);
+static void count(const char *c, size_t l, void *p);
 static void counts2(const char *c, void *p);
-static void save(char c, void *p);
+static void save(const char *c, size_t l, void *p);
 static void saves2(const char *c, void *p);
 
 char *rfc2047_encode_header_addr(const struct rfc822a *a,
@@ -216,42 +217,48 @@ char *rfc2047_encode_header_tobuf(const char *name, /* Header name */
 	return rfc2047_encode_str(header, charset, rfc2047_qp_allow_word);
 }
 
-static void count(char c, void *p)
+static void count(const char *c, size_t l, void *p)
 {
-	++*(size_t *)p;
+	*(size_t *)p += l;
 }
 
 static void counts2(const char *c, void *p)
 {
 	if (*c == ',')
-		count(*c++, p);
+	{
+		count(c, 1, p);
+		++c;
+	}
 
-	count('\n', p);
-	count(' ', p);
+	count("\n ", 2, p);
 
-	while (*c)	count(*c++, p);
+	count(c, strlen(c), p);
 }
 
-static void save(char c, void *p)
+static void save(const char *c, size_t l, void *p)
 {
-	**(char **)p=c;
-	++*(char **)p;
+	memcpy(*(char **)p, c, l);
+	*(char **)p += l;
 }
 
 static void saves2(const char *c, void *p)
 {
 	if (*c == ',')
-		save(*c++, p);
+	{
+		save(c, 1, p);
+		++c;
+	}
 
-	save('\n', p);
-	save(' ', p);
+	save("\n ", 2, p);
 
-	while (*c)	save(*c++, p);
+	save(c, strlen(c), p);
 }
 
 static int encodebase64(const char *ptr, size_t len, const char *charset,
 			int (*qp_allow)(char),
-			int (*func)(const char *, size_t, void *), void *arg)
+			int (*func)(const char *, size_t, void *),
+			int *inappropriate,
+			void *arg)
 {
 	unsigned char ibuf[3];
 	char obuf[4];
@@ -307,7 +314,9 @@ static int encodebase64(const char *ptr, size_t len, const char *charset,
 static int encodeqp(const char *ptr, size_t len,
 		    const char *charset,
 		    int (*qp_allow)(char),
-		    int (*func)(const char *, size_t, void *), void *arg)
+		    int (*func)(const char *, size_t, void *),
+		    int *inappropriate,
+		    void *arg)
 {
 	size_t i;
 	int rc;
@@ -343,6 +352,14 @@ static int encodeqp(const char *ptr, size_t len,
 			rc=(*func)("_", 1, arg);
 		else
 		{
+			if (i == 0)
+			{
+				/* The preceding char was ? so we can't follow
+				** it with a = */
+				*inappropriate=1;
+				return 0;
+			}
+
 			buf[0]='=';
 			buf[1]=xdigit[ ( ptr[i] >> 4) & 0x0F ];
 			buf[2]=xdigit[ ptr[i] & 0x0F ];
@@ -452,8 +469,10 @@ static int do_encode_words_method(const char32_t *uc,
 						 int (*qp_allow)(char),
 						 int (*func)(const char *,
 							     size_t, void *),
+						 int *inappropriate,
 						 void *arg),
 				  int (*func)(const char *, size_t, void *),
+				  int *inappropriate,
 				  void *arg)
 {
 	char    *p;
@@ -504,7 +523,7 @@ static int do_encode_words_method(const char32_t *uc,
 			--psize;
 
 		rc=(*encoder)(p, psize, charset, qp_allow,
-			      func, arg);
+			      func, inappropriate, arg);
 		free(p);
 		if (rc)
 			return rc;
@@ -538,7 +557,8 @@ static int do_encode_words(const char32_t *uc,
 	size_t  psize;
 	int rc;
 	size_t b64len, qlen;
-
+	int inappropriate_q;
+	int inappropriate_b64;
 	/*
 	** Convert from unicode
 	*/
@@ -565,31 +585,40 @@ static int do_encode_words(const char32_t *uc,
 	*/
 	qlen=0;
 	b64len=0;
+	inappropriate_q=0;
+	inappropriate_b64=0;
 
 	rc=do_encode_words_method(uc, ucsize, charset, qp_allow, offset,
-				  &encodeqp, cnt_conv, &qlen);
+				  &encodeqp, cnt_conv,
+				  &inappropriate_q,
+				  &qlen);
 	if (rc)
 		return rc;
 
 	rc=do_encode_words_method(uc, ucsize, charset, qp_allow, offset,
-				  &encodebase64, cnt_conv, &b64len);
+				  &encodebase64, cnt_conv,
+				  &inappropriate_b64, &b64len);
 	if (rc)
 		return rc;
 
 	return do_encode_words_method(uc, ucsize, charset, qp_allow, offset,
-				      qlen < b64len ? encodeqp:encodebase64,
-				      func, arg);
+				      (qlen < b64len
+				       && inappropriate_q == 0)
+				      ? encodeqp:encodebase64,
+				      func,
+				      &inappropriate_q, // Doesn't matter
+				      arg);
 }
 
 /*
 ** RFC2047-encoding pass.
 */
-static int rfc2047_encode_callback(const char32_t *uc,
-				   size_t ucsize,
-				   const char *charset,
-				   int (*qp_allow)(char),
-				   int (*func)(const char *, size_t, void *),
-				   void *arg)
+int rfc2047_encode_callback(const char32_t *uc,
+			    size_t ucsize,
+			    const char *charset,
+			    int (*qp_allow)(char),
+			    int (*func)(const char *, size_t, void *),
+			    void *arg)
 {
 	int	rc;
 	size_t	i;
