@@ -2,7 +2,7 @@
 #define	rfc822_h
 
 /*
-** Copyright 1998 - 2025 Double Precision, Inc.
+** Copyright 1998 - 2025 S. Varshavchik.
 ** See COPYING for distribution information.
 */
 
@@ -1397,7 +1397,10 @@ class fdstreambuf : public std::streambuf {
 	char *defaultbuf{nullptr};
 
 public:
-	fdstreambuf(int fd=-1) noexcept : fd{fd} {}
+
+	fdstreambuf() noexcept : fdstreambuf(-1) {}
+
+	explicit fdstreambuf(int fd) noexcept : fd{fd} {}
 
 	fdstreambuf(fdstreambuf &&o) noexcept
 	{
